<?php
include "restutils.php";
class Person {
	/**
	 * Return the list of people in the community
	 * @success 200
	 * @format text/json
	 * @example 
	 * [
	 *    {
	 *       "id":"/officemon/resources/person/xyz@acme.com",
	 *       "name":"xyz",
	 *       "email":"xyz@acme.com",
	 *       "home_phone":"3434343",
	 *       "reason": 
	 *       {
	 *          "AM":"SICK",
	 *          "PM":"WAH"
	 *       },
	 *       "details":
	 *       {
	 *          "AM": "cold",
	 *          "PM": "recovering"
	 *       },
	 *       "absenceid":"23432432434"
	 *    }
	 * ]
	 *
	 */
	function onList() {
		$dm = data_manager('hallmondb');

		$date = zget('/request/params/date', "");
		
		$teamName = zget('/request/params/team', null);
		
		$personEmail = zget('/request/params/email', null);
		
		if ($teamName != null) {
			$sql = "select people.name as name, people.email as email , people.home_phone as home_phone, absence.tod as tod, absence.reason as reason , absence.details as details, absence.updatestamp as absenceid from people inner join teammembers on teammembers.name = ? and email = member_email left join absence on people.email = absence.email and startdate <= CURRENT_DATE and enddate >= CURRENT_DATE ORDER BY people.name, updatestamp desc";
			$people = data_exec($dm, $sql, array($teamName));
		} else if ($personEmail !=  null) {
			$sql = 'select people.name as name, people.email as email , people.home_phone as home_phone, absence.tod as tod, absence.reason as reason , absence.details as details, absence.updatestamp as absenceid from people left join absence on people.email = absence.email and startdate <= CURRENT_DATE and enddate >= CURRENT_DATE where people.email = ? ORDER BY  updatestamp desc';
			$people = data_exec($dm, $sql, array($personEmail));
		} else {
			$sql = 'select people.name as name, people.email as email , people.home_phone as home_phone, absence.tod as tod, absence.reason as reason , absence.details as details, absence.updatestamp as absenceid from people left join absence on people.email = absence.email and startdate <= CURRENT_DATE and enddate >= CURRENT_DATE ORDER BY people.name, updatestamp desc';
			$people = data_exec($dm, $sql);
		}

		$peoplecopy = $this->summarizeAbsence($people);


		zput('/request/headers/out/Content-Type', 'text/json');
		echo json_encode($peoplecopy);
	}
	
	/**
	 * Create a new user in the community
	 * @success 204 User created in the community
	 * @error 400 Problem with data supplied
	 * @format text/json
	 * @example 
	 * {
	 *     "name":"xyz",
	 *     "email":"xyz@acme.com",
	 *     "home_phone":"34324342"
	 * }
	 */
	function onCreate() {
		$result = false;
		$person = json_decode($HTTP_RAW_POST_DATA);
		
		if ($this->valid_person($person)) {

			$dm = data_manager('hallmondb');
			// TODO: check for valid person
			$result = data_exec($dm, 'INSERT INTO people (name, email, home_phone)'.
						'VALUES (?, ?, ?)', 
						array($person['name'], $person['email'], $person['home_phone']));
			if ($result == 1){ // Set a Location header with URI to the new record
				$locationUri = zget("/request/path") . "/" . $person["email"];
				zput("/request/headers/out/Location", $locationUri);
				zput("/request/status", 204);
				return;
			} 
		}
		// If we get here there was an error somewhere
		echo "Problem with data/application";
		zput('/request/status', 400);	
	}
	
	/**
	 * Get the details of a particular person in the community
	 * @success 200 details of the person
	 * @error 404 person with email address was not found
	 * @format text/json
	 * @example 
	 * {
	 *     "name":"xyz",
	 *     "email":"xyz@acme.com",
	 *     "phone":"234343434",
	 *     "teams":
	 *     [
	 *        {
	 *            "name": "security",
	 *            "id": "/officemon/resources/team/security/xyz@acme.com"
	 *        }
	 *     ]
	 *     "notifications":
	 *     [
	 *        {
	 *            "name":"abc@acme.com",
	 *            "id": "/officemon/resources/emailnotification/xyz@acme.com/abc@acme.com"
	 *        }
	 *     ]
	 *     "teamnotifications":
	 *     [
	 *        {
	 *            "name":"security",
	 *            "id":"/officemon/resources/teamnotification/xyz@acme.com/security"
	 *        }
	 *     ]
	 * }
	 */
	function onRetrieve() {
		$email = zget('/request/params/personId');
		$contextRoot = zget('/config/contextRoot', "");

		$dm = data_manager('hallmondb');
		//$person = data_query_first($dm, "SELECT * FROM people WHERE email = ?", array($email));
		
		$sql = 'select people.name as name, people.email as email , people.home_phone as home_phone, absence.tod as tod, absence.reason as reason , absence.details as details, absence.updatestamp as absenceid from people left join absence on people.email = absence.email and startdate <= CURRENT_DATE and enddate >= CURRENT_DATE where people.email = ? ORDER BY  updatestamp desc';
		$people = data_exec($dm, $sql, array($email));
		$peoplecopy = $this->summarizeAbsence($people);
		$person = $peoplecopy[0];

		if (isset($person)) {
	
			// Get the teams that this person belongs to ?
			$teamsresult = data_exec($dm, "SELECT name FROM teammembers WHERE member_email = ?", array($email));
			// Convert to appropriate form
			$teams = array();
			foreach ($teamsresult as $teamselem) {
				$membershipid = $contextRoot."/resources/team/".$teamselem['name']."/$email";
				$teams[] = array('name' => $teamselem['name'], 'id' => $membershipid);
			}
			$person['teams'] = $teams;

			// Get list of people whose absence this member wishes to be notifed about
			$notifyresults = data_exec($dm, "SELECT absent_person from notifications WHERE notified_person = ?",
				array($email));
			// Convert to appropriate form
			$notifications = array();
			foreach ($notifyresults as $notification) {
				// **** Hardcoded and hand crafted id ****
				$notificationId = $contextRoot."/resources/emailnotification/$email/".$notification['absent_person'];
				$notifications[] = array('name' => $notification['absent_person'], 'id' => $notificationId);
			}
			$person['notifications'] = $notifications;
			
			$teamnotifyresults = data_exec($dm, "SELECT team_name as teamname from teamnotifications WHERE notified_person = ?",
				array($email));
			$teamnotifications = array();
			foreach($teamnotifyresults as $notification) {
				$notificationId = $contextRoot."/resources/teamnotification/$email/".$notification['teamname'];
				$teamnotifications[] = array('name' => $notification['teamname'], 'id' => $notificationId);
			}
			$person['teamnotifications'] = $teamnotifications;

			zput('/request/headers/out/Content-Type', 'text/json');
			echo json_encode($person);
		} else {
			zput('/request/status', 404);
			zput('/request/error', "Person with email address ($email) was not found");
		}
	}
	
	/**
	 * Update the person in the community
	 * @success 204 Person update correctly
	 * @error 400 Bad data supplied
	 * @format text/json
	 * @example 
	 * {
	 *     "name":"xyz",
	 *     "email":"xyz@acme.com",
	 *     "home_phone":"234343434"
	 * }
	 */
	function onUpdate() {
	
		$personemail = zget('/request/params/personId');
		$person = json_decode($HTTP_RAW_POST_DATA);
		$dm = data_manager('hallmondb');

		// Update the fields that have changed
		// TODO: More efficient if we use iterator and a loop - avoids excessive lookups
		$sql = "UPDATE people SET ";
		$sqlargs = array();
		$sqldelim = '';
		if ($person['name'] != NULL) {
			$sql .= 'name=? ';
			$sqlargs[] = $person['name'];
			$sqldelim = ',';
		}
		if ($person['home_phone'] != NULL) {
			$sql .= $sqldelim.' home_phone=? ';
			$sqlargs[] = $person['home_phone'];
			$sqldelim = ',';	
		}
		if ($person['email'] != NULL) {
			$sql .= $sqldelim. ' email=? ';
			$sqlargs[] = $person['email'];
		}


		if (count($sqlargs) > 0) {
			$sql .= ' WHERE email=? ';
			$sqlargs[] = $personemail;
			$result = data_exec($dm, $sql, $sqlargs);
		}

		if ($result == TRUE) {	
			zput("/request/status", 204);
		} else {
			//TODO:  Need user understandable error message from data query
			zput("/request/status", 400);
		}
	}
	
	/**
	 * Delete the person specified from the community
	 * @success 204 Person deleted from the community and from any other relationships (teams/notifications)
	 * @error 404 person with email address was not found
	 * 
	 *
	 */
	function onDelete() {
		$email = zget('/request/params/personId');
		$dm = data_manager('hallmondb');
		// Delete this person from any teams that he may be a member of
		$person = data_query_first($dm, "SELECT * FROM people WHERE email = ?", array($email));

		if (isset($person)) {

			$result = data_exec($dm, "DELETE FROM teammembers WHERE member_email = ?", array($email));

			// Delete notifications to or from this person
			$result = data_exec($dm, "DELETE FROM notifications WHERE notified_person = ? OR absent_person = ?",
			array($email, $email));

			// Delete team notifications to this person
			$result = data_exec($dm, "DELETE FROM teamnotifications WHERE notified_person = ?", array($email));

			// Delete person
			$result = data_exec($dm, "DELETE FROM people WHERE email=?", array($email));

			zput("/request/status", 204);
		}else{
			zput("/request/status", 404);	 // Not found.
			zput('/request/error', "Person with email address ($email) was not found");
		}
	}
	
	/**
	 * Validate JSON input
	 * Invalid if email or name does not exist
	 * @param array $person 
	 */
	function valid_person($person) {
		$valid = false;
		if (isset($person['name']) &&
			isset($person['email']) &&
			isset($person['home_phone']) &&
			trim($person['name']) != "" &&
			trim($person['email']) != "") {
			
			$valid = true;
		}
		return $valid;
	}
	
	function summarizeAbsence($people) {
		/*
		 * Eliminate duplicates and create array with AM and PM status for each person
		 * How this is supposed to work ?
		 * We get a list of statuses for all the people in the organization. This list may have duplicates depending
 		 * on the number of overlapping status changes for a particular person. The list is ordered by the most recent 
		 * update first.
		 * In general if there is no status update for a person, then we are done for that person (there wont be duplicates).
		 * If the most recent status update was an all day update, we are done in that case as well.
		 * The complexity arises if the most recent update is a partial day (AM or PM) status update. In this case, we continue processing status updates till
		 * we can find the update for a user that will complete the update. 
		 * 
		 * The status is reflected in the reason and details field, these are converted to arrays keyed by either 'AM' or 'PM' or both.
		 * 
		 */
		$last_email = NULL;
		$last_tod = NULL;
		$status_found = true;
		$peoplecopy = array();
		$uriprefix = generate_uriprefix();
		$processed = 0;
		foreach ($people as $person) {
			if ($status_found) {
				// if we are still at the person that was last processed continue
				if ($last_email == $person['email']) {
					continue;
				} else {
					// New person - start processing
					$status_found = false;
					$last_email = $person['email'];
					$last_tod = NULL;
					$processed++;
				}
			}
			// The tod field is either NULL (no absence information) or AM, PM or ALL (indicates some sort of absence)
			if ($person['tod'] == NULL) {
				// copy and continue
				$person['id'] = $uriprefix.$person['email'];
				unset($person['tod']);
				$peoplecopy[] = $person;
				$status_found = true;
			} else if ($person['tod'] == 'ALL') {
				// Is this the first status for this person
				if ($last_tod == NULL) { 
					// put the same reason and details for AM and PM
					$reason = $person['reason'];
					$details = $person['details'];
					// convert reason and details into an array
					$person['reason'] = array('AM' => $reason, 'PM' => $reason);
					$person['details'] = array('AM' => $details, 'PM' => $details);
					unset($person['tod']);
					$person['id'] = $uriprefix.$person['email'];
					$person['absence'] = array();
					$person['absence'][] = array('id' => $person['absenceid'], 'summary' => $person['details']);
					$peoplecopy[] = $person;
				} else {
					// We found partial status earlier. Update person with complete status
					if ($last_tod == 'AM') {
						$current_tod = 'PM';
					} else {
						$current_tod = 'PM';
					}
					$updatePerson = $peoplecopy[$processed-1];
					if ($updatePerson['email'] != $person['email']) {
						// Assertion failure
						echo "Failed to meet assertion for completing status(ALL) :". $updatePerson['email']."!=".$person['email'];
						return;
					}
					$updatePerson['reason'][$current_tod] = $person['reason'];
					$updatePerson['details'][$current_tod] = $person['details'];
					$updatePerson['absence'][] = array('id' => $person['absenceid'], 'summary' => $person['details']);
					$peoplecopy[$processed - 1] = $updatePerson;
				}
				$status_found = true;		
			} else {
				// We got an absence status for either AM or PM
				// Is this the first status for this person	
				if ($last_tod == NULL) {
					$last_tod = $person['tod'];
					$reason = $person['reason'];
					$details = $person['details'];
					// Convert reason and details into an array
					$person['reason'] = array($last_tod => $reason);
					$person['details'] = array($last_tod => $details);
					unset($person['tod']);
					$person['id'] = $uriprefix.$person['email'];
					$person['absence'] = array();
					$peoplecopy[] = $person;
				} else {
					// We found partial status earlier. Update the person with complete status
					if ($last_tod == 'AM') {
						$current_tod = 'PM';
					} else {
						$current_tod = 'PM';
					}
					$updatePerson = $peoplecopy[$processed-1];
					if ($updatePerson['email'] != $person['email']) {
						// Assertion failure
						echo "Failed to meet assertion for completing status(AM|PM) :". $updatePerson['email']."!=".$person['email'];
						return;
					}
					$updatePerson['reason'][$current_tod] = $person['reason'];
					$updatePerson['details'][$current_tod] = $person['details'];
					$updatePerson['absence'][] = array('id' => $person['absenceid'], 'summary' => $person['details']);
					$peoplecopy[$processed-1] = $updatePerson;
					$status_found = true;
				}
			}
	
		}
		return $peoplecopy;
	}
	
}

?>