<?php
include "restutils.php";
class Team {
	
	/**
	 * List all the teams
	 * @success 200 returns the list of teams
	 * @format text/json
	 * @example 
	 * [
	 *    {
	 *       "name":"core",
	 *       "description": "The CORE team",
	 *       "membercount": "10"
	 *    }
	 * ]
	 */
	function onList() {
		$defaultOrg = 'Zero';
		$dm = data_manager('hallmondb');
		$teams = data_exec($dm, "SELECT name, description FROM teams WHERE orgname = ?", array($defaultOrg));

		$uriprefix = generate_uriprefix();
		$teamcopy = array();
		foreach ($teams as $team) {
			$members = data_exec($dm, "SELECT COUNT(*) FROM teammembers WHERE name=?", array($team['name']));
			$team['id'] = $uriprefix.$team['name'];
			// This is really odd !
			$team['membercount'] = $members[0][1];
			$teamcopy[] = $team;
		}
		zput('/request/headers/out/Content-Type', 'text/json');
		echo json_encode($teamcopy);
	}
	
	/**
	 * Create a new team
	 * @success 204 team successfully created
	 * @error 400 Problem with data (team already exists)
	 * @format text/json
	 * @example 
	 * {
	 *     "name": "core",
	 *     "description": "The Core team"
	 * }
	 *
	 */
	function onCreate() {
		$team = json_decode($HTTP_RAW_POST_DATA);
		$defaultOrg = 'Zero';

		$dm = data_manager('hallmondb');
		// TODO: check for valid person
		$result = data_exec($dm, 'INSERT INTO teams (name, description, orgname)'.
				'VALUES (?, ?, ?)', 
				array($team['name'], $team['description'], $defaultOrg));

		if ($result == 1) {
			// Set a Location header with URI to the new record
			$locationUri = zget("/request/path") . "/" . $team["name"];
			zput("/request/headers/out/Location", $locationUri);
			zput("/request/status", 204);
		} else {
			zput('/request/errormsg', "Could not create team");
			zput('/request/status', 400);
		}
		
	}
	
	/**
	 * Retreive a particular team's details
	 * @success 200 returns the team details
	 * @error 404 team not found
	 * @format text/json
	 * @example
	 * {
	 *    "name":"core",
	 *    "description":"The core team",
	 *    "members": 
	 *    [
	 *       "xyz@acme.com",
	 *       "abc@acme.com"
	 *    ]
	 * }
	 */
	function onRetrieve() {
		$teamname = zget('/request/params/teamId');
		$return_memberlist = zcontains('/request/params/memberlist');

		$dm = data_manager('hallmondb');
		if (!$return_memberlist) {
			// return the team info with a members email array
			$team = data_query_first($dm, "SELECT * FROM teams WHERE name = ?", array($teamname));

			if (isset($team)) {
				// Get the members of the team
	
				$memberresult = data_exec($dm, "SELECT * FROM teammembers WHERE name = ?",	array($teamname));
				// convert to indexed array
				if (isset($memberresult)) {
					$members = array();
					foreach ($memberresult as $memberelem) {
						$members[] = $memberelem['member_email'];
					}
					$team['members'] = $members;
				}
				zput('/request/headers/out/Content-Type', 'text/json');
				echo json_encode($team);
			} else {
				zput('/request/status', 404);
				zput('/request/error', "Team with name ($team) was not found");
			}
		} else { // return the memberlist as a collection
			$members = data_exec($dm, "SELECT people.name, member_email FROM people INNER JOIN teammembers ON teammembers.name = ? AND teammembers.member_email = people.email ",
							array($teamname));
			$uriprefix = generate_uriprefix();
			$memberscopy = array();
			foreach ($members as $member) {
				// TODO: Fix member id to point to another resource
				$member['id'] = $uriprefix.$member['member_email'];
				$memberscopy[] = $member;
			}
			echo json_encode($memberscopy);
	
		}
	}
	
	/**
	 * Update the team resource
	 * @success 204 team details updated
	 * @error 400 problem with data supplied
	 * @format text/json
	 * @example
	 * {
	 *    "name": "core",
	 *    "description": "THE CORE TEAM",
	 *    "members":
	 *    [
	 *       "def@acme.com"
	 *    ]
	 *    "removedmembers":
	 *    [
	 *       "abc@acme.com"
	 *    ]
	 * }
	 *
	 */
	function onUpdate() {
		
		$teamname = zget('/request/params/teamId');
		$team = json_decode($HTTP_RAW_POST_DATA);
		$dm = data_manager('hallmondb');
		$sql = "UPDATE teams SET ";
		$result = FALSE;
		$sqlargs = array();
		$sqldelim = '';
		if (isset($team['name'])) {
			$sql .= 'name=? ';
			$sqlargs[] = $team['name'];
			$sqldelim = ',';
		}
		if (isset($team['description'])) {
			$sql .= $sqldelim.' description=? ';
			$sqlargs[] = $team['description'];	
		}

		if (count($sqlargs) > 0) {
			$sql .= ' WHERE name=? ';
			$sqlargs[] = $teamname;
			$result = data_exec($dm, $sql, $sqlargs);
		}
		// If the team name was changed, update the team name in the following sql
		if ($result == TRUE && isset($team['name']) && $team['name'] != NULL) {
			// Update the teammembers table if the team name changed.
			data_exec($dm, "UPDATE teammembers SET name=? WHERE name=?", array($team['name'], $teamname));
			$teamname = $team['name'];
		}

		$updateFailed = false;
		// Update teammembers table if there are new members
		if (isset($team['members'])) {
			foreach ($team['members'] as $member_email){
				$updates = data_exec($dm, "INSERT INTO teammembers (name, member_email)".
					" VALUES(?, ?)", array($teamname, $member_email));
				if ($updates != 1) {
					$updateFailed = true;
				}
			}
		}
		// Update team if members have been removed
		if (isset($team['removedmembers'])) {
			foreach ($team['removedmembers'] as $member_email){
				$updates = data_exec($dm, "DELETE FROM teammembers WHERE name = ? AND member_email = ?", 
					array($teamname, $member_email));
				if ($updates != 1) {
					$updateFailed = true;
				}
			}
		}
		
		if ($updateFailed) {
			zput('/request/status', 400);
		} else {
			zput("/request/status", 204);
		}
	}
	
	/**
	 * Delete a team and its relationships (membership info)
	 * @success 204 team successfully deleted
	 * @error 400 problem with data
	 */
	function onDelete() {
		$team = zget('/request/params/teamId');
		$member = zget('/event/pathInfo', NULL);
		$deleteTeam = true;
		if ($member != NULL) {
			$deleteTeam = false;
			$member = substr($member, 1);
		}
		$dm = data_manager('hallmondb');
		$result = 0;
		if ($deleteTeam) {
			// DELETE from team members
			$result += data_exec($dm, "DELETE from teammembers where name = ?", array($team));
			// DELETE from team notifications
			$result += data_exec($dm, "DELETE from teamnotifications where team_name = ?", array($team));
			// Delete from teams
			$result += data_exec($dm, "DELETE FROM teams WHERE name=?", array($team));
		} else {
			$result += data_exec($dm, "DELETE FROM teammembers WHERE name = ? AND member_email = ?", 
					array($team, $member));
		}
		if ($result > 0) {
			zput("/request/status", 204);
		} else {
			zput('/request/status', 400);
		}
	}
	
}
?>